@@ -35,10 +35,13 @@ use tokio::{
 use tracing::{debug, info};
 use xor_name::XorName;
 
+pub use register_apis::RegisterWriteAheadLog;
+
 /// Client object
 #[derive(Clone, Debug)]
 pub struct Client {
     keypair: Keypair,
+    #[allow(dead_code)]
     incoming_errors: Arc<RwLock<Receiver<CmdError>>>,
     session: Session,
     pub(crate) query_timeout: Duration,
