@@ -12,12 +12,9 @@ mod chaos;
 /// Configuration handling
 pub mod config_handler;
 mod error;
-mod event_mapping;
 mod logging;
-mod metadata;
 mod network;
 mod node_api;
-mod node_ops;
 
 /// Docs
 pub mod state_db;
