@@ -391,7 +391,7 @@ impl NetworkKnowledge {
     }
 
     // Try to merge this `NetworkKnowledge` members with `peers`. .
-    async fn merge_members(&self, peers: SectionPeers) -> Result<bool> {
+    pub(crate) async fn merge_members(&self, peers: SectionPeers) -> Result<bool> {
         let mut there_was_an_update = false;
         let chain = self.chain.read().await.clone();
 
